(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/forms'), require('rxjs'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('ng-codemirror', ['exports', '@angular/core', '@angular/forms', 'rxjs', '@angular/common'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ng-codemirror'] = {}, global.ng.core, global.ng.forms, global.rxjs, global.ng.common));
}(this, (function (exports, core, forms, rxjs, common) { 'use strict';

    var CodeMirrorComponent = /** @class */ (function () {
        function CodeMirrorComponent(elementRef, ngZone, renderer, _differs) {
            this.elementRef = elementRef;
            this.ngZone = ngZone;
            this.renderer = renderer;
            this._differs = _differs;
            this.value = null;
            this.onTouchedCallback = function () { };
            this.onChangeCallback = function () { };
            this.autoMaxHeight = 0;
            this.delayRefreshTime = 200;
            this.focusChange = new core.EventEmitter();
            this.codemirrorClassName = true;
        }
        Object.defineProperty(CodeMirrorComponent.prototype, "codeMirrorGlobal", {
            get: function () {
                if (this._codeMirror) {
                    return this._codeMirror;
                }
                this._codeMirror = typeof CodeMirror !== 'undefined' ? CodeMirror : require('codemirror');
                return this._codeMirror;
            },
            enumerable: false,
            configurable: true
        });
        CodeMirrorComponent.prototype.ngOnInit = function () {
            var _this = this;
            if (!this._differ && this.options) {
                this._differ = this._differs.find(this.options).create();
            }
            this.initializeCodemirror();
            rxjs.timer(this.delayRefreshTime).subscribe(function () {
                if (_this.autoMaxHeight) {
                    _this.initializeAutoMaxHeight();
                }
                _this.editor.refresh();
            });
        };
        CodeMirrorComponent.prototype.ngOnChanges = function (simpleChanges) {
            var _this = this;
            var optionsChange = simpleChanges.options;
            if (optionsChange && !optionsChange.firstChange) {
                var changes = this._differ.diff(this.options);
                if (changes) {
                    changes.forEachChangedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
                    changes.forEachAddedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
                    changes.forEachRemovedItem(function (option) { return _this.setOptionIfChanged(option.key, option.currentValue); });
                }
            }
        };
        CodeMirrorComponent.prototype.registerOnChange = function (fn) {
            this.onChangeCallback = fn;
        };
        CodeMirrorComponent.prototype.registerOnTouched = function (fn) {
            this.onTouchedCallback = fn;
        };
        CodeMirrorComponent.prototype.writeValue = function (value) {
            var valid = value !== null && value !== undefined;
            if (this.editor && valid && this.value !== value) {
                this.editor.setValue(value);
            }
            this.value = value;
        };
        CodeMirrorComponent.prototype.initializeCodemirror = function () {
            var _this = this;
            this.ngZone.runOutsideAngular(function () {
                if (!_this.options) {
                    throw new Error("options is required");
                }
                _this.editor = _this.codeMirrorGlobal.fromTextArea(_this.textAreaRef.nativeElement, _this.options);
                _this.editor.on("focus", function () { return _this.ngZone.run(function () { return _this.focusChange.emit(true); }); });
                _this.editor.on("blur", function () {
                    _this.ngZone.run(function () { return _this.focusChange.emit(false); });
                    _this.editor.setCursor(0, null, { scroll: false });
                });
                _this.editor.on("change", function (cm, change) {
                    if (change.origin !== "setValue") {
                        _this.value = cm.getValue();
                        _this.onChangeCallback(_this.value);
                    }
                });
            });
        };
        CodeMirrorComponent.prototype.initializeAutoMaxHeight = function () {
            var _this = this;
            this._codeWrapElement = this.elementRef.nativeElement.querySelector(".CodeMirror-code");
            this._codemirrorContentObserver = new MutationObserver(function () {
                _this.applyEditorHeight();
            });
            this._codemirrorContentObserver.observe(this._codeWrapElement, {
                childList: true,
            });
        };
        CodeMirrorComponent.prototype.applyEditorHeight = function () {
            if (this._codeWrapElement.offsetHeight >= this.autoMaxHeight) {
                var wrapHeight = this.elementRef.nativeElement.querySelector(".CodeMirror");
                if (wrapHeight.offsetHeight > this.autoMaxHeight) {
                    this.editor.setSize("100%", this.autoMaxHeight + "px");
                }
            }
            else {
                this.editor.setSize("100%", "auto");
            }
        };
        CodeMirrorComponent.prototype.setOptionIfChanged = function (optionName, newValue) {
            if (!this.editor) {
                return;
            }
            // cast to any to handle strictly typed option names
            // could possibly import settings strings available in the future
            this.editor.setOption(optionName, newValue);
        };
        CodeMirrorComponent.prototype.ngOnDestroy = function () {
            if (this._codemirrorContentObserver) {
                this._codemirrorContentObserver.disconnect();
            }
        };
        return CodeMirrorComponent;
    }());
    CodeMirrorComponent.decorators = [
        { type: core.Component, args: [{
                    selector: "ng-codemirror, [ngCodeMirror]",
                    template: "<textarea #textAreaRef style=\"display:none;\"></textarea>",
                    preserveWhitespaces: false,
                    changeDetection: core.ChangeDetectionStrategy.OnPush,
                    providers: [{
                            provide: forms.NG_VALUE_ACCESSOR,
                            useExisting: core.forwardRef(function () { return CodeMirrorComponent; }),
                            multi: true
                        }]
                },] }
    ];
    CodeMirrorComponent.ctorParameters = function () { return [
        { type: core.ElementRef },
        { type: core.NgZone },
        { type: core.Renderer2 },
        { type: core.KeyValueDiffers }
    ]; };
    CodeMirrorComponent.propDecorators = {
        autoMaxHeight: [{ type: core.Input }],
        options: [{ type: core.Input }],
        delayRefreshTime: [{ type: core.Input }],
        focusChange: [{ type: core.Output }],
        textAreaRef: [{ type: core.ViewChild, args: ["textAreaRef", { read: core.ElementRef, static: true },] }],
        codemirrorClassName: [{ type: core.HostBinding, args: ["class.ng-codemirror",] }]
    };

    var CodemirrorModule = /** @class */ (function () {
        function CodemirrorModule() {
        }
        return CodemirrorModule;
    }());
    CodemirrorModule.decorators = [
        { type: core.NgModule, args: [{
                    imports: [common.CommonModule],
                    declarations: [CodeMirrorComponent],
                    entryComponents: [CodeMirrorComponent],
                    exports: [CodeMirrorComponent]
                },] }
    ];

    /*
     * Public API Surface of ng-codemirror
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.CodeMirrorComponent = CodeMirrorComponent;
    exports.CodemirrorModule = CodemirrorModule;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ng-codemirror.umd.js.map
